
using System;
using System.Threading; 
using System.Collections.Generic;
using System.Text;
using System.Data;
using GanttChart.Model; 
using GanttChart.DataAccess.DDSInterface; 

namespace GanttChart.DataAccess
{
    public class COM_Component
    {

        private Guid _mostRecentlyParsedProgramGuid;
        public Guid mostRecentlyParsedProgramGuid
        {
            get { return _mostRecentlyParsedProgramGuid; }
            set { _mostRecentlyParsedProgramGuid = value; }
        }

        private Guid _mostRecentlyParsedPhaseGuid;
        public Guid mostRecentlyParsedPhaseGuid
        {
            get { return _mostRecentlyParsedPhaseGuid; }
            set { _mostRecentlyParsedPhaseGuid = value;}
        }

        private DateTime _mostRecentlyParsedPhaseStartTime;
        public DateTime mostRecentlyParsedPhaseStartTime
        {
            get { return _mostRecentlyParsedPhaseStartTime; }
            set { _mostRecentlyParsedPhaseStartTime = value; }
        }

        // Publishers
        public GanttChart.DataAccess.DDSInterface.CreateProgramRequestTypePublisher createProgramRequestTypePublisher;
        public GanttChart.DataAccess.DDSInterface.DefinePhaseRequestTypePublisher definePhaseRequestTypePublisher;
        public GanttChart.DataAccess.DDSInterface.DefineTaskRequestTypePublisher defineTaskRequestTypePublisher;

        // Subscribers
        public GanttChart.DataAccess.DDSInterface.CreateProgramRequestTypeSubscriber createProgramRequestTypeSubscriber;
        public GanttChart.DataAccess.DDSInterface.DefinePhaseRequestTypeSubscriber definePhaseRequestTypeSubscriber;
        public GanttChart.DataAccess.DDSInterface.DefineTaskRequestTypeSubscriber defineTaskRequestTypeSubscriber;

        public COM_Component()
        {
            // Create publishers
            createProgramRequestTypePublisher = new CreateProgramRequestTypePublisher();
            definePhaseRequestTypePublisher = new DefinePhaseRequestTypePublisher();
            defineTaskRequestTypePublisher = new DefineTaskRequestTypePublisher();
        }

        public void StartListen()
        {
            createProgramRequestTypeSubscriber = new CreateProgramRequestTypeSubscriber(this);
            definePhaseRequestTypeSubscriber = new DefinePhaseRequestTypeSubscriber(this);
            defineTaskRequestTypeSubscriber = new DefineTaskRequestTypeSubscriber(this);
        }

        public void OnDataReceived(CreateProgramRequestType msg)
        {
            DataRow newRow = ProgramRepository.Instance.theTasks.NewRow();
            Guid thisGuid = Guid.NewGuid();
            newRow["TaskGuid"] = thisGuid;
            mostRecentlyParsedProgramGuid = thisGuid;
            newRow["PlanID"] = ProgramRepository.Instance.defaultProjectName;
            string temp = new string(msg.ProgramPlanUniqueID.Value);
            newRow["TaskName"] = temp; 
            newRow["ParentTaskGuid"] = null;                        // Program tasks never have parents in the Gantt Chart
            newRow["TaskStartTime"] = DateTime.UtcNow;
            newRow["TaskDuration"] = TimeSpan.FromMinutes(1);
            newRow["TaskPercentComplete"] = 0;
            newRow["TaskType"] = GanttChart.Model.TaskType.Program;
            ProgramRepository.Instance.theTasks.Rows.Add(newRow);
        }

        public void OnDataReceived(DefinePhaseRequestType msg)
        {
            DataRow newRow = ProgramRepository.Instance.theTasks.NewRow();
            Guid thisGuid = Guid.NewGuid();
            newRow["TaskGuid"] = thisGuid;
            mostRecentlyParsedPhaseGuid = thisGuid;
            newRow["PlanID"] = ProgramRepository.Instance.defaultProjectName;
            newRow["TaskName"] = new string(msg.phase.phaseName);
            newRow["ParentTaskGuid"] = mostRecentlyParsedProgramGuid;
            mostRecentlyParsedPhaseStartTime = ConvertToDateTime(msg.phaseParameters.phaseStartTime);
            DateTime finishTime = ConvertToDateTime(msg.phaseParameters.phaseFinishTime);
            TimeSpan duration = finishTime - mostRecentlyParsedPhaseStartTime;
            newRow["TaskStartTime"] = mostRecentlyParsedPhaseStartTime;
            newRow["TaskDuration"] = duration;
            newRow["TaskPercentComplete"] = 0;
            newRow["TaskType"] = GanttChart.Model.TaskType.Phase;
            ProgramRepository.Instance.theTasks.Rows.Add(newRow);
        }

        public void OnDataReceived(DefineTaskRequestType msg)
        {
            DataRow newRow = ProgramRepository.Instance.theTasks.NewRow();
            Guid thisGuid = Guid.NewGuid();
            newRow["TaskGuid"] = thisGuid;
            newRow["PlanID"] = ProgramRepository.Instance.defaultProjectName;
            newRow["TaskName"] = new string(msg.task.taskName);
            newRow["ParentTaskGuid"] = mostRecentlyParsedPhaseGuid;  // DefineTaskRequestTypes always have phases as parents in the Gantt Chart
            DateTime specificStartTime = mostRecentlyParsedPhaseStartTime.AddSeconds(msg.taskParameters.startDelay);
            newRow["TaskStartTime"] = specificStartTime;
            newRow["TaskDuration"] = TimeSpan.FromDays(msg.taskParameters.dwellTime / 86400);  //convert seconds to days
            newRow["TaskPercentComplete"] = 0;
            newRow["TaskPercentComplete"] = 0;
            newRow["TaskType"] = GanttChart.Model.TaskType.BaseTask;
            ProgramRepository.Instance.theTasks.Rows.Add(newRow);
        }

        DateTime ConvertToDateTime(double unixTime)
        {
            var epoch = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
            return epoch.AddSeconds(unixTime);
        }
    }
}
