using System;
using System.Collections.Generic;
using System.Text;
using UCS.AMT; 
using DDS;
using GanttChart.Model;

namespace GanttChart{

namespace DataAccess{

    namespace DDSInterface
    {

        public class DefinePhaseRequestTypePublisher
        {

            public DefinePhaseRequestTypePublisher()
            {
                //// --- Set domain ID --- //
                int domain_id = 0;

                /* Uncomment this to turn on additional logging
                NDDS.ConfigLogger.get_instance().set_verbosity_by_category(
                    NDDS.LogCategory.NDDS_CONFIG_LOG_CATEGORY_API, 
                    NDDS.LogVerbosity.NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
                */

                // --- Create participant --- //

                /* To customize participant QoS, use 
                   the configuration file USER_QOS_PROFILES.xml */
                participant =
                    DDS.DomainParticipantFactory.get_instance().create_participant(
                        domain_id,
                        DDS.DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT,
                        null /* listener */,
                        DDS.StatusMask.STATUS_MASK_NONE);
                if (participant == null)
                {
                    shutdown();
                    throw new ApplicationException("create_participant error");
                }

                // --- Create publisher --- //

                /* To customize publisher QoS, use 
                   the configuration file USER_QOS_PROFILES.xml */
                DDS.Publisher publisher = participant.create_publisher(
                DDS.DomainParticipant.PUBLISHER_QOS_DEFAULT,
                null /* listener */,
                DDS.StatusMask.STATUS_MASK_NONE);
                if (publisher == null)
                {
                    shutdown();
                    throw new ApplicationException("create_publisher error");
                }

                // --- Create topic --- //

                /* Register type before creating topic */
                System.String type_name = DefinePhaseRequestTypeTypeSupport.get_type_name();
                try
                {
                    DefinePhaseRequestTypeTypeSupport.register_type(
                        participant, type_name);
                }
                catch (DDS.Exception e)
                {
                    Console.WriteLine("register_type error {0}", e);
                    shutdown();
                    throw e;
                }

                /* To customize topic QoS, use 
                   the configuration file USER_QOS_PROFILES.xml */
                DDS.Topic topic = participant.create_topic(
                    "UCS::AMT::DefinePhaseRequestTopic::AMT",
                    type_name,
                    DDS.DomainParticipant.TOPIC_QOS_DEFAULT,
                    null /* listener */,
                    DDS.StatusMask.STATUS_MASK_NONE);
                if (topic == null)
                {
                    shutdown();
                    throw new ApplicationException("create_topic error");
                }

                // --- Create writer --- //

                /* To customize data writer QoS, use 
                   the configuration file USER_QOS_PROFILES.xml */
                DDS.DataWriter writer = publisher.create_datawriter(
                    topic,
                    DDS.Publisher.DATAWRITER_QOS_DEFAULT,
                    null /* listener */,
                    DDS.StatusMask.STATUS_MASK_NONE);
                if (writer == null)
                {
                    shutdown();
                    throw new ApplicationException("create_datawriter error");
                }
                DefinePhaseRequestType_writer = (DefinePhaseRequestTypeDataWriter)writer;
            }


            public void publish(UCS.AMT.DefinePhaseRequestType instance)
            {
                /* For a data type that has a key, if the same instance is going to be
                   written multiple times, initialize the key here
                   and register the keyed instance prior to writing */
                DDS.InstanceHandle_t instance_handle = DDS.InstanceHandle_t.HANDLE_NIL;
                //instance_handle = DefinePhaseRequestType_writer.register_instance(instance);

                /* Modify the data to be sent here */
                try
                {
                    DefinePhaseRequestType_writer.write(instance, ref instance_handle);
                }
                catch (DDS.Exception e)
                {
                    Console.WriteLine("write error {0}", e);
                }

                System.Threading.Thread.Sleep(500);

                /*
                try {
                    DefinePhaseRequestType_writer.unregister_instance(
                        instance, ref instance_handle);
                } catch(DDS.Exception e) {
                    Console.WriteLine("unregister instance error: {0}", e);
                }
                */
            }

            public void shutdown()
            {
                // --- Shutdown --- //
                /* Delete all entities */

                if (participant != null)
                {
                    participant.delete_contained_entities();
                    DDS.DomainParticipantFactory.get_instance().delete_participant(
                        ref participant);
                }

                /* RTI Connext provides finalize_instance() method on
                   domain participant factory for people who want to release memory
                   used by the participant factory. Uncomment the following block of
                   code for clean destruction of the singleton. */
                /*
                try {
                    DDS.DomainParticipantFactory.finalize_instance();
                } catch (DDS.Exception e) {
                    Console.WriteLine("finalize_instance error: {0}", e);
                    throw e;
                }
                */
            }

            private DDS.DomainParticipant participant;
            private DefinePhaseRequestTypeDataWriter DefinePhaseRequestType_writer;
        }

} /* namespace DDSInterface */

} /* namespace DataAccess */

} /* namespace GanttChart */
