using System;
using System.Collections.Generic;
//using System.Data; 
using System.Text; 
//using GanttChart.Model;
using UCS.AMT;

namespace GanttChart
{

namespace DataAccess
{

namespace DDSInterface
{

public class CreateMissionRequestTypeSubscriber {

    public class CreateMissionRequestTypeListener : DDS.DataReaderListener {

        public override void on_requested_deadline_missed(
            DDS.DataReader reader,
            ref DDS.RequestedDeadlineMissedStatus status) {}
    
        public override void on_requested_incompatible_qos(
            DDS.DataReader reader,
            DDS.RequestedIncompatibleQosStatus status) {}
    
        public override void on_sample_rejected(
            DDS.DataReader reader,
            ref DDS.SampleRejectedStatus status) {}

        public override void on_liveliness_changed(
            DDS.DataReader reader,
            ref DDS.LivelinessChangedStatus status) {}

        public override void on_sample_lost(
            DDS.DataReader reader,
            ref DDS.SampleLostStatus status) {}

        public override void on_subscription_matched(
            DDS.DataReader reader,
            ref DDS.SubscriptionMatchedStatus status) {}

        public override void on_data_available(DDS.DataReader reader) {
            CreateMissionRequestTypeDataReader CreateMissionRequestType_reader =
                (CreateMissionRequestTypeDataReader)reader;
            
            // take
            try {
                CreateMissionRequestType_reader.take(
                    data_seq,
                    info_seq,
                    DDS.ResourceLimitsQosPolicy.LENGTH_UNLIMITED,
                    DDS.SampleStateKind.ANY_SAMPLE_STATE,
                    DDS.ViewStateKind.ANY_VIEW_STATE,
                    DDS.InstanceStateKind.ANY_INSTANCE_STATE);
            }
            catch(DDS.Retcode_NoData) {
                return;
            }
            catch(DDS.Exception e) {
                Console.WriteLine("take error {0}", e);
                return;
            }
            // notify
            System.Int32 data_length = data_seq.length;
            for (int i = 0; i < data_length; ++i) {
                if (info_seq.get_at(i).valid_data) {
                    this._component.OnDataReceived(data_seq.get_at(i));
                    //CreateMissionRequestTypeTypeSupport.print_data(data_seq.get_at(i));
                }
            }

            // return loan
            try {
                CreateMissionRequestType_reader.return_loan(data_seq, info_seq);
            }
            catch(DDS.Exception e) {
                Console.WriteLine("return loan error {0}", e);
            }
        }

        public CreateMissionRequestTypeListener(COM_Component component) {
            data_seq = new CreateMissionRequestTypeSeq();
            info_seq = new DDS.SampleInfoSeq();
            _component = component; 
        }

        private CreateMissionRequestTypeSeq data_seq;
        private DDS.SampleInfoSeq info_seq;
        private COM_Component _component; 
    };

    public CreateMissionRequestTypeSubscriber(COM_Component component)
    {
        //// --- Set domain ID --- //
        int domain_id = 0;

        // --- Create participant --- // 

        /* To customize the participant QoS, use 
           the configuration file USER_QOS_PROFILES.xml */
        participant = DDS.DomainParticipantFactory.get_instance().create_participant(
                domain_id,
                DDS.DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT, 
                null /* listener */,
                DDS.StatusMask.STATUS_MASK_NONE);
        if (participant == null) {
            shutdown(participant);
            throw new ApplicationException("create_participant error");
        }

        // --- Create subscriber --- //

        /* To customize the subscriber QoS, use 
           the configuration file USER_QOS_PROFILES.xml */
        DDS.Subscriber subscriber = participant.create_subscriber(
            DDS.DomainParticipant.SUBSCRIBER_QOS_DEFAULT,
            null /* listener */,
            DDS.StatusMask.STATUS_MASK_NONE);
        if (subscriber == null) {
            shutdown(participant);
            throw new ApplicationException("create_subscriber error");
        }

        // --- Create topic --- //

        /* Register the type before creating the topic */
        System.String type_name = CreateMissionRequestTypeTypeSupport.get_type_name();
        try {
            CreateMissionRequestTypeTypeSupport.register_type(
                participant, type_name);
        }
        catch(DDS.Exception e) {
            Console.WriteLine("register_type error {0}", e);
            shutdown(participant);
            throw e;
        }

        /* To customize the topic QoS, use 
           the configuration file USER_QOS_PROFILES.xml */
        DDS.Topic topic = participant.create_topic(
            "Example CreateMissionRequestType",
            type_name,
            DDS.DomainParticipant.TOPIC_QOS_DEFAULT,
            null /* listener */,
            DDS.StatusMask.STATUS_MASK_NONE);
        if (topic == null) {
            shutdown(participant);
            throw new ApplicationException("create_topic error");
        }

        // --- Create reader --- //

        /* Create a data reader listener */
        reader_listener =
            new CreateMissionRequestTypeListener(component);

        /* To customize the data reader QoS, use 
           the configuration file USER_QOS_PROFILES.xml */
        DDS.DataReader reader = subscriber.create_datareader(
            topic,
            DDS.Subscriber.DATAREADER_QOS_DEFAULT,
            reader_listener,
            DDS.StatusMask.STATUS_MASK_ALL);
        if (reader == null) {
            shutdown(participant);
            reader_listener = null;
            throw new ApplicationException("create_datareader error");
        }

    }

    ~CreateMissionRequestTypeSubscriber()
    {
        /* Delete all entities */
        shutdown(participant);
        reader_listener = null;
    }


    public static void Listen()
    {
        // --- Wait for data --- //

        /* Main loop */
        const System.Int32 receive_period = 4000; // milliseconds
        while (true) 
        {
            System.Threading.Thread.Sleep(receive_period);
        }
    }


    static void shutdown( DDS.DomainParticipant participant ) {

        /* Delete all entities */

        if (participant != null) {
            participant.delete_contained_entities();
            DDS.DomainParticipantFactory.get_instance().delete_participant(
                ref participant);
        }

        /* RTI Connext provides finalize_instance() method on
           domain participant factory for users who want to release memory
           used by the participant factory. Uncomment the following block of
           code for clean destruction of the singleton. */
        
        try 
        {
            DDS.DomainParticipantFactory.finalize_instance();
        }
        catch(DDS.Exception e) {
            Console.WriteLine("finalize_instance error {0}", e);
            throw e;
        }
    }

    private DDS.DomainParticipant participant;
    private CreateMissionRequestTypeListener reader_listener; 
}

} /* namespace DDSInterface */

} /* namespace DataAccess */

} /* namespace GanttChart */