﻿using System;
using System.Data;
using System.Collections.Generic;
using GanttChart.DataAccess;
using GanttChart.DataAccess.DDSInterface;

namespace GanttChart.Model
{
    enum TaskType { Program, Phase, BaseTask }; 

    public class ProgramRepository
    {
        //All
        public COM_Component component;
        public String defaultProjectName = "db";
        public DataSet programPlanRepo;
        public DataTable theProgramPlans;
        public DataTable theTasks;

        private static ProgramRepository _instance = null;
        private static readonly object _locker = new object();

        private ProgramRepository()
        {
            InitializeRepos();
            lock (_locker)
            {
                component = new COM_Component();
            }
        }

        public static ProgramRepository Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (_locker)
                    {
                        if (_instance == null)
                        {
                            _instance = new ProgramRepository();
                            _instance.component.StartListen();
                        }
                    }
                }
                return _instance;
            }
        }

        public void InitializeRepos()
        {
            //All vehicles
            #region

            lock (_locker)
            {
                programPlanRepo = new DataSet();
                theProgramPlans = programPlanRepo.Tables.Add("ProgramPlans");
                theProgramPlans.Columns.Add("PlanGuid");
                theProgramPlans.Columns.Add("PlanID");
                theProgramPlans.Columns.Add("PlanName");
                theProgramPlans.Columns.Add("PlanStartTime", typeof(DateTime));

                //set the primary key to the vehicleID Column
                DataColumn[] keys = new DataColumn[1];
                keys[0] = theProgramPlans.Columns["PlanID"];
                theProgramPlans.PrimaryKey = keys;

                theTasks = programPlanRepo.Tables.Add("Tasks");
                theTasks.Columns.Add("TaskGuid");
                theTasks.Columns.Add("PlanID");
                theTasks.Columns.Add("TaskName");
                theTasks.Columns.Add("TaskStartTime", typeof(DateTime));
                theTasks.Columns.Add("TaskDuration", typeof(TimeSpan));
                theTasks.Columns.Add("ParentTaskGuid");
                theTasks.Columns.Add("TaskPercentComplete");
                theTasks.Columns.Add("TaskType");    // Stores whether this is a Program, phase, or base task
                // Any other Task properties not covered above are stored as Binary data in the AllProperties field. 
                // This includes properties like Resources, Predecessors.
                theTasks.Columns.Add("AllProperties", typeof(Byte[]));

                //set the primary key to the tasksGuid Column
                DataColumn[] tasksKey = new DataColumn[1];
                tasksKey[0] = theTasks.Columns["TaskGuid"];
                theTasks.PrimaryKey = tasksKey;

                System.Data.DataRelation relOrdDet;
                System.Data.DataColumn colMaster;
                System.Data.DataColumn colDetail;
                colMaster = programPlanRepo.Tables["ProgramPlans"].Columns["PlanID"];
                colDetail = programPlanRepo.Tables["Tasks"].Columns["PlanID"];
                relOrdDet = new System.Data.DataRelation("RelOrdDet", colMaster, colDetail);
                programPlanRepo.Relations.Add(relOrdDet);

                // Now create a default project workspace
                DataRow newRow = theProgramPlans.NewRow();
                Guid thisGuid = Guid.NewGuid();
                newRow["PlanGuid"] = thisGuid;
                newRow["PlanID"] = defaultProjectName;
                newRow["PlanName"] = defaultProjectName;
                newRow["PlanStartTime"] = DateTime.Now;
                theProgramPlans.Rows.Add(newRow);
            }
            #endregion
        }
    }
}