/*********************************************************************************/
/*                                                                               */
/* Title: BindingGanttView.cs                                                           */
/*                                                                               */
/* SBIR Data Rights                                                              */
/*                                                                               */
/* Contract No. : FA8650-10-M-6088                                               */
/*                                                                               */
/* Contractor Name: Kutta Technologies, Inc.                                     */
/* Address: 2075 W. Pinnacle Peak Rd., Suite 102, Phoenix, AZ 85027-1219         */
/*                                                                               */
/* Copyright:                                                                    */
/*    Expiration of SBIR Data Rights Period: In accordance with DFARS            */
/*    252.227-7018, expiration of SBIR Data Rights is five years after           */
/*    completion of the project, including any follow-on phases.                 */
/*                                                                               */
/*    The Government's rights to use, modify, reproduce, release, perform,       */
/*    display, or disclose technical data or computer software marked with       */
/*    this legend are restricted during the period shown as provided in          */
/*    paragraph (b)(4) of the Rights in Noncommercial Technical Data and         */
/*    Computer Software--Small Business Innovative Research (SBIR) Program       */
/*    clause contained in the above identified contract. No restrictions         */
/*    apply after the expiration date shown above. Any reproduction of           */
/*    technical data, computer software, or portions thereof marked with this    */
/*    legend must also reproduce the markings.                                   */
/*                                                                               */

/* -------- ---------------------------------------------------------------      */ 
/*                                                                               */
/*********************************************************************************/


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Diagnostics; 
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinSchedule;
using Infragistics.Win.UltraWinSchedule.TaskUI;
using Infragistics.Win.UltraWinEditors; 
using Infragistics.Win.UltraWinGanttView;
using Infragistics.Win.UltraWinTabControl; 
using GanttChart.Model;
using GanttChart.View;

namespace GanttChart.ViewModel
{
    public partial class BindingGanttView : Form
    {
 
        #region Constructor

        public BindingGanttView()
        {
            InitializeComponent();
        }


        #endregion

        #region Private Methods


        #endregion

        #region Form Load Event

        private void BindingGanttView_Load(object sender, EventArgs e)
        {

            //  Set the BindingContextControl property to reference this form
            this.ultraCalendarInfo1.DataBindingsForTasks.BindingContextControl = this;
            this.ultraCalendarInfo1.DataBindingsForProjects.BindingContextControl = this;

            // Set the DataSource and DataMember for Project and Tasks using the SetDataBinding method
            this.ultraCalendarInfo1.DataBindingsForProjects.SetDataBinding(MissionRepository.Instance.battlePlanRepo, "BattlePlans");
            this.ultraCalendarInfo1.DataBindingsForTasks.SetDataBinding(MissionRepository.Instance.battlePlanRepo, "Tasks");
            this.ultraCalendarInfo1.DataBindingsForOwners.SetDataBinding(MissionRepository.Instance.battlePlanRepo, "Vehicles");

            //  Set the DataBinding members for Projects 
            this.ultraCalendarInfo1.DataBindingsForProjects.IdMember = "PlanGuid";
            this.ultraCalendarInfo1.DataBindingsForProjects.KeyMember = "PlanID";
            this.ultraCalendarInfo1.DataBindingsForProjects.NameMember = "PlanName";
            this.ultraCalendarInfo1.DataBindingsForProjects.StartDateMember = "PlanStartTime";

            //  Set the DataBinding members for Tasks
            // Basic Task properties 
            this.ultraCalendarInfo1.DataBindingsForTasks.NameMember = "TaskName"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.DurationMember = "TaskDuration"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.StartDateTimeMember = "TaskStartTime"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.IdMember = "TaskGuid"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.ProjectKeyMember = "PlanID"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.ParentTaskIdMember = "ParentTaskGuid";
            this.ultraCalendarInfo1.DataBindingsForTasks.PercentCompleteMember = "TaskPercentComplete";
            // All other properties
            this.ultraCalendarInfo1.DataBindingsForTasks.AllPropertiesMember = "AllProperties";


            //  Set the DataBinding members for Vehicles/TaskOwners
            // Basic Task properties 
            this.ultraCalendarInfo1.DataBindingsForOwners.KeyMember = "VehicleID";
            this.ultraCalendarInfo1.DataBindingsForOwners.NameMember = "VehicleName"; 

            //  Since we are showing a task that belongs to an explicitly defined
            //  project (i.e., not the UnassignedProject), assign that project to
            //  the control's Project property so the control knows to display that project.
            this.ultraGanttView1.CalendarInfo = this.ultraCalendarInfo1;
            Project project = new Project(); 
            project.Key = MissionRepository.Instance.defaultProjectName; 
            project.Name = MissionRepository.Instance.defaultProjectName; 
            project.StartDate = DateTime.Now; 
            this.ultraCalendarInfo1.Projects.Add(project);
            this.ultraGanttView1.Project = this.ultraGanttView1.CalendarInfo.Projects[1]; 
            
            this.ultraGanttView1.CreationFilter = new CurrentDayIndicatorCreationFilter();
            this.ultraGanttView1.GridAreaWidth =
                20 + this.ultraGanttView1.GridSettings.ColumnSettings[TaskField.Name].Width
                + this.ultraGanttView1.GridSettings.ColumnSettings[TaskField.Duration].Width
                + this.ultraGanttView1.GridSettings.ColumnSettings[TaskField.StartDateTime].Width
                + this.ultraGanttView1.GridSettings.ColumnSettings[TaskField.EndDateTime].Width
                + this.ultraGanttView1.GridSettings.ColumnSettings[TaskField.Resources].Width
                + this.ultraGanttView1.GridSettings.ColumnSettings[TaskField.Dependencies].Width; 

            ultraGrid1.DataSource = MissionRepository.Instance.battlePlanRepo;
            ultraGrid1.DataMember = "Tasks"; 
            
        }

        #endregion

        #region //Event Handlers
        private void ultraGanttView1_MessageReceived(object sender, EventArgs e)
        {
        }

        private void ultraGanttView1_Click(object sender, EventArgs e)
        {
        }

        public void ultraGanttView1_PropertyChanged(object sender, Infragistics.Win.PropertyChangedEventArgs e)
        {
        }

        public void ultraGanttView1_ContextMenuInitializing(object sender, Infragistics.Win.UltraWinGanttView.ContextMenuInitializingEventArgs e)
        {
            /*try
            {
                if (e.Area == Infragistics.Win.UltraWinGanttView.ContextMenuArea.Grid)
                {
                    ContextMenu customMenu = null;
                    if (e.Task == null)
                        return;
                    if ((e.Task.Parent == null)) // Treat task is a mission
                    {
                        customMenu = (new ContextMenuMission(ultraGanttView1.ActiveTask, ultraGanttView1.CalendarInfo.Tasks, ultraGanttView1) as ContextMenu);
                    }
                    else if (e.Task.Parent.Parent == null) // Treat task is a Phase
                    {
                    }
                    else // Treat task is a base task
                    {
                    }
                    e.ContextMenu = customMenu;
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error in ultraGanttView1_TaskDialogDisplaying");
                Debug.WriteLine(ex.GetType().Name);
                Debug.WriteLine(ex.Message);
            }
            finally
            {
                //e.Cancel = true;  // Cancels the default ContextMenu
            }*/
        }
        
        public void ultraGanttView1_TaskDialogDisplaying(object sender, Infragistics.Win.UltraWinGanttView.TaskDialogDisplayingEventArgs e)
        {
            try
            {
                TaskDialog infragisticsDialog = null;
                if (e.Task.Parent != null)
                {
                    // Process Task entries
                    if (e.Task.Parent.Parent != null) 
                    {
                        // Create the basic task form
                        infragisticsDialog = new TaskDialog(e.Task, e.Task.Parent.Tasks, ultraGanttView1);
                        infragisticsDialog.Text = "Task Entry Dialog";
    
                        // Add custom task form to it
                        UltraTabControl targetTab = infragisticsDialog.Controls[0] as UltraTabControl;
                        DialogTaskEntry dialogTaskEntry = new DialogTaskEntry();
                        System.Drawing.Size newSize = new System.Drawing.Size(infragisticsDialog.Width, infragisticsDialog.Height + dialogTaskEntry.Height); 
                        infragisticsDialog.FindForm().MinimumSize = newSize;
                        targetTab.Tabs["General"].TabPage.Controls.Add(dialogTaskEntry);
                    }
                    // Process Phase entries
                    else 
                    {
                        infragisticsDialog = new TaskDialog(e.Task, e.Task.Parent.Tasks, ultraGanttView1);
                        infragisticsDialog.Text = "Phase Entry Dialog";

                        UltraTabControl targetTab = infragisticsDialog.Controls[0] as UltraTabControl;
                        DialogPhaseEntry dialogPhaseEntry = new DialogPhaseEntry();
                        System.Drawing.Size newSize = new System.Drawing.Size(infragisticsDialog.Width, infragisticsDialog.Height + dialogPhaseEntry.Height); 
                        infragisticsDialog.FindForm().MinimumSize = newSize; 
                        targetTab.Tabs["General"].TabPage.Controls.Add(dialogPhaseEntry);
                    }
                }
                // Process Mission entries
                else 
                {
                    infragisticsDialog = new TaskDialog(ultraGanttView1.ActiveTask, ultraGanttView1.CalendarInfo.Tasks, ultraGanttView1);
                    infragisticsDialog.Text = "Mission Entry Dialog"; 
                }
                infragisticsDialog.ShowDialog();
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error in ultraGanttView1_TaskDialogDisplaying");
                Debug.WriteLine(ex.GetType().Name);
                Debug.WriteLine(ex.Message);
            }
            finally
            {
                e.Cancel = true;  // Cancels the default Task Dialog Form
            }
        }

        public void ultraGanttView1_TaskDataAdded(object sender, TaskAddedEventArgs e)
        {
        }

        public void ultraGanttView1_TaskAdded(object sender, TaskAddedEventArgs e)
        {
            e.Task.SetDuration(e.Task.Duration, TimeSpanFormat.Minutes);
        }

        private void ultraCalendarInfo1_TaskDataInitialized(object sender, TaskDataInitializedEventArgs e)
        {

            try
            {
                e.Task.SetDuration(e.Task.Duration, TimeSpanFormat.Minutes);
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error changing task duration from days to minutes");
                Debug.WriteLine(ex.GetType().Name);
                Debug.WriteLine(ex.Message);
            }
        }

        #endregion //Event Handlers
    }
}