
﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinGanttView;

using nsTLV = Infragistics.Win.UltraWinSchedule.TimelineView;
using nsGVUI = Infragistics.Win.UltraWinGanttView.UIElements;

namespace GanttChart.ViewModel
{
    #region CurrentDayIndicatorCreationFilter
    /// <summary>
    /// IUIElementCreationFilter implementation used to draw a vertical line
    /// alongside the beginning of the first time slot for the current date.
    /// </summary>
    public class CurrentDayIndicatorCreationFilter : IUIElementCreationFilter
    {
        #region Members
        private VerticalLineUIElement verticalLineElement = null;
        private AppearanceBase appearance = null;
        #endregion Members

        #region IUIElementCreationFilter

        void IUIElementCreationFilter.AfterCreateChildElements(UIElement parent)
        {
           DateTime currentTimeInterval;
            //  Get a reference to the UIElement which contains the time slot elements.
            nsGVUI.TaskAreaUIElement taskAreaElement = parent as nsGVUI.TaskAreaUIElement;

            if ( taskAreaElement != null )
            {
                //  Use GetDescendant to find the time slot for the current date
               currentTimeInterval = getCurrentTimeInterval(DateTime.Now);
                nsTLV.TimeSlotUIElement currentDateFirstSlot =
                    taskAreaElement.GetDescendant( typeof(nsTLV.TimeSlotUIElement), currentTimeInterval ) as
                    nsTLV.TimeSlotUIElement;

                if (currentDateFirstSlot != null)
                {
                   //  Create the Appearance if its null
                   if (this.appearance == null)
                   {
                      this.appearance = new Infragistics.Win.Appearance();

                      //  There are a million and one orange Office2k7 colors, use one of them
                      this.appearance.BorderColor = Color.Red;
                   }

                   //  Create the VerticalLineUIElement if its null, initialize it if its not null
                   if (this.verticalLineElement == null)
                      this.verticalLineElement = new VerticalLineUIElement(taskAreaElement, UIElementBorderStyle.Solid, this.appearance);
                   else
                      this.verticalLineElement.Initialize(UIElementBorderStyle.Solid, this.appearance);

                   //  Set the bounds of the VerticalLineUIElement
                   Rectangle rect = currentDateFirstSlot.Rect;
                   rect.Width = DrawUtility.CalculateBorderWidths(UIElementBorderStyle.Solid).Left;
                   this.verticalLineElement.Rect = rect;

                   //  Add the VerticalLineUIElement to the TaskAreaUIElement's ChildElements collection.
                   taskAreaElement.ChildElements.Add(this.verticalLineElement);
                }
                else
                {
                   Console.WriteLine("currentDateFirstSlot is null for " + currentTimeInterval);
                }
            }
        }

        bool IUIElementCreationFilter.BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }

        DateTime getCurrentTimeInterval(DateTime now)
        {
           DateTime derivedTime;
           /*if (now.Minute <= 30)
           {
              derivedTime = new DateTime(now.Year, now.Month, now.Day,now.Hour, 30, 0);
           }
           else
           {
              derivedTime = new DateTime(now.Year, now.Month, now.Day,now.Hour+1, 0, 0);
           }*/
           derivedTime = new DateTime(now.Year, now.Month, now.Day,now.Hour, now.Minute, 0);
           return derivedTime;
        }
        #endregion IUIElementCreationFilter
    }
    #endregion CurrentDayIndicatorCreationFilter

    #region VerticalLineUIElement
    /// <summary>
    /// Simple UIElement which draws a vertical line with a particular
    /// border style and border color(s).
    /// </summary>
    public class VerticalLineUIElement : UIElement
    {
        #region Constants
        public static readonly Color DefaultBorderColor = Color.Orange;
        public static readonly Color DefaultBorderColor2 = Color.Transparent;
        #endregion Constants

        #region Members
        private UIElementBorderStyle borderStyle = UIElementBorderStyle.Solid;
        private AppearanceBase appearance = null;
        #endregion Members

        #region Constructor
        /// <summary>
        /// Creates a new instance.
        /// </summary>
        public VerticalLineUIElement(
            UIElement parentElement,
            UIElementBorderStyle borderStyle,
            AppearanceBase appearance ) :
            base( parentElement )
        {
            this.Initialize( borderStyle, appearance );
        }

        /// <summary>
        /// Recycles this instance.
        /// </summary>
        public void Initialize( UIElementBorderStyle borderStyle, AppearanceBase appearance )
        {
            this.borderStyle = borderStyle;
            this.appearance = appearance;
        }
        #endregion Constructor

        #region BorderSides
        /// <summary>
        /// Only draw the left border.
        /// </summary>
        public override Border3DSide BorderSides
        {
            get { return Border3DSide.Left; }
            set { throw new NotSupportedException(); }
        }
        #endregion BorderSides

        #region BorderStyle
        /// <summary>
        /// Returns the border style for this element.
        /// </summary>
        public override UIElementBorderStyle BorderStyle
        {
            get { return this.borderStyle; }
            set { throw new NotSupportedException(); }
        }
        #endregion BorderStyle

        #region InitAppearance
        /// <summary>
        /// Initializes the appearance data for this element.
        /// </summary>
        protected override void InitAppearance(ref AppearanceData appearance, ref AppearancePropFlags requestedProps)
        {
            Color borderColor = this.appearance != null ? this.appearance.BorderColor : VerticalLineUIElement.DefaultBorderColor;
            Color borderColor2 = this.appearance != null ? this.appearance.BorderColor2 : VerticalLineUIElement.DefaultBorderColor2;

            #region BorderColor
            if ( (requestedProps & AppearancePropFlags.BorderColor) == AppearancePropFlags.BorderColor &&
                 appearance.HasPropertyBeenSet(AppearancePropFlags.BorderColor) == false )
            {
                appearance.BorderColor = borderColor;
                requestedProps &= ~AppearancePropFlags.BorderColor;
            }
            #endregion BorderColor

            #region BorderColor2
            if ( (requestedProps & AppearancePropFlags.BorderColor2) == AppearancePropFlags.BorderColor2 &&
                 appearance.HasPropertyBeenSet(AppearancePropFlags.BorderColor2) == false )
            {
                appearance.BorderColor2 = borderColor2;
                requestedProps &= ~AppearancePropFlags.BorderColor2;
            }
            #endregion BorderColor2
        }
        #endregion InitAppearance

        #region WantsInputNotification
        /// <summary>
        /// Returns false so that this element does notrespond to user interaction.
        /// </summary>
        protected override bool WantsInputNotification(UIElementInputType inputType, Point point)
        {
            return false;
        }
        #endregion WantsInputNotification
    }
    #endregion VerticalLineUIElement
}